context("ClimProjDiags::SelBox tests")

##############################################

# dat1
dat1 <- 1:(2 * 3 * 2 * 4)
dim(dat1) <- c(lon = 2, lat = 3, time = 2, model = 4)
lon <- seq(2, 40, 20)
lat <- c(1, 5, 10)
region <-  c(1, 2, 1, 1)
mask <- array(c(0, 1, 0, 1, 0, 1), dim = c(lon = 2, lat = 3))

# dat2
dat2 <- 1:(2 * 3 * 2 * 4)
dim(dat2) <- c(lons = 2, lats = 3, time = 2, model = 4)
mask2 <- array(c(0, 1, 0, 1, 0, 1), dim = c(lons = 2, lats = 3))

# dat3
dat3 <- 1:(20 * 3)
dim(dat3) <- c(longitude = 20, latitude = 3)
lon3 <- seq(2, 40, 2)
lat3 <- c(1, 5, 10)
region3 <- c(2, 20, 1, 5)
region3_2 <- c(15, 5, 1, 2)
mask3 <- array(c(0, 1, 0, 1, 0, 1), dim = c(latitude = 3, longitude = 20))

##############################################

test_that("1. Input checks", {
  # data
  expect_error(
    SelBox(c(), lon = 1:10, lat = 1:10),
    "Parameter 'data' cannot be NULL."
  )
  expect_error(
    SelBox(1:10, lon = 1:10, lat = 1:10),
    "Parameter 'data' must be a numeric array."
  )
  expect_error(
    SelBox(array(1:10), lon = 1:10, lat = 1:10),
    "Parameter 'data' must have dimension names."
  )
  # lon, lat
  expect_error(
    SelBox(array(1:10, dim = c(lon = 10)), lon = NULL, lat = 1:10),
    "Parameters 'lon' and 'lat' cannot be NULL."
  )
  expect_error(
    SelBox(array(1:10, dim = c(lon = 10)), lon = 'a', lat = 1:10),
    "Parameters 'lon' and 'lat' must be numeric."
  )
  expect_error(
    SelBox(array(1:10, dim = c(lon = 10)), 
           lon = 1:10, lat = array(1:10, dim = c(lon = 10, lat = 1))),
    "Parameters 'lon' and 'lat' need to be a vector."
  )
  # region
  expect_error(
    SelBox(dat1, lat = lat, lon = lon, region = NULL),
    "Parameter 'region' cannot be NULL."
  )
  expect_error(
    SelBox(dat1, lat = lat, lon = lon, region = 'a'),
    "Parameter 'region' must be numeric."
  )
  expect_error(
    SelBox(dat1, lat = lat, lon = lon, region = 1:3),
    paste0("The region argument has to be a vector of length four indicating the ",
           "minimum longitude, the maximum longitude, the minimum latitude and ", 
           "the maximum latitude.")
  )
  # londim
  expect_warning(
    SelBox(array(1:10, dim = c(lon = 10, lat = 1)), region = region,
           lon = 1:10, lat = 1, londim = 1, latdim = 2),
    "Numeric 'londim' is deprecated, use dimension names instead. The ", 
    "corresponding dimension name will be assigned."
  )
  expect_error(
    SelBox(array(1:10, dim = c(lon = 10, lat = 1)), region = region, 
           lon = 1:10, lat = 1, londim = list(1,2), latdim = 2),
    "Parameter 'londim' must be a character string."
  )
  expect_warning(
    SelBox(array(1:10, dim = c(lon = 10, lat = 1)), region = region, 
           lon = 1:10, lat = 1, londim = c('lon', 'lat'), latdim = 'lat'),
    "Parameter 'londim' must be of length 1. Only the first value ", 
    "will be used."
  )
  expect_error(
    SelBox(array(1:10, dim = c(lons = 10, lat = 1)), region = region, 
           lon = 1:10, lat = 1, londim = 'lon', latdim = 'lat'),
    "Parameter 'londim' is not found in 'data'."
  )
  expect_error(
    SelBox(array(1:10, dim = c(lon = 10, lat = 1)), region = region, 
           lon = 1:11, lat = 1, londim = 'lon', latdim = 'lat'),
    paste0("The longitudinal dimension of parameter 'data' must be of the ", 
           "same length as parameter 'lon'.")
  )
  # latdim
  expect_warning(
    SelBox(array(1:10, dim = c(lon = 10, lat = 1)), region = region, 
           lon = 1:10, lat = 1, latdim = 2),
    "Numeric 'latdim' is deprecated, use dimension names instead. The ", 
    "corresponding dimension name will be assigned."
  )
  expect_error(
    SelBox(array(1:10, dim = c(lon = 10, lat = 1)), region = region, 
           lon = 1:10, lat = 1, latdim = list(1,2)),
    "Parameter 'latdim' must be a character string."
  )
  expect_warning(
    SelBox(array(1:10, dim = c(lon = 10, lat = 1)), region = region, 
           lon = 1:10, lat = 1, latdim = c('lat', 'lon')),
    "Parameter 'latdim' must be of length 1. Only the first value ", 
    "will be used."
  )
  expect_error(
    SelBox(array(1:10, dim = c(lon = 10, lats = 1)), region = region, 
           lon = 1:10, lat = 1, londim = 'lon', latdim = 'lat'),
    "Parameter 'latdim' is not found in 'data'."
  )
  expect_error(
    SelBox(array(1:10, dim = c(lon = 10, lat = 1)), region = region, 
           lon = 1:10, lat = 1:2, londim = 'lon', latdim = 'lat'),
    paste0("The latitudinal dimension of parameter 'data' must be of the ", 
           "same length as parameter 'lat'.")
  )
  # mask
  expect_error(
    SelBox(dat1, lat = lat, lon = lon, region = region, mask = 1:3),
    "Parameter 'mask' must have the same spatial dimensions of data."
  )
  expect_error(
    SelBox(dat1, lat = lat, lon = lon, region = region, mask = mask2),
    "Parameter 'mask' must have the same spatial dimensions of data."
  )
})

##############################################

test_that("2. Output checks", {
  # dat2
  expect_equal(
    names(SelBox(dat2, lat = lat, lon = lon, region = region, 
                 londim = 'lons', latdim = 'lats', mask = mask2)), 
    c('data', 'lon', 'lat', 'mask')
  )
  expect_equal(
    dim(SelBox(dat2, lat = lat, lon = lon, region = region, 
               londim = 'lons', latdim = 'lats')$data), 
    c(lons = 1, lats = 1, time = 2, model = 4)
  )
  expect_equal(
    dim(SelBox(dat2, lat = lat, lon = lon, region = region, 
               londim = 'lons', latdim = 'lats', mask = mask2)$mask), 
    c(lons = 1, lats = 1)
  )
  expect_equal(
    as.vector(SelBox(dat2, lat = lat, lon = lon, region = region, 
              londim = 'lons', latdim = 'lats', mask = mask2)$data), 
    c(1, 7, 13, 19, 25, 31, 37, 43)
  )
  expect_equal(
    SelBox(dat2, lat = lat, lon = lon, region = region, 
           londim = 'lons', latdim = 'lats', mask = mask2)$lat, 
    1
  )
  # dat3
  expect_equal(
    dim(SelBox(dat3, lat = lat3, lon = lon3, region = region3, 
               londim = 'longitude', latdim = 'latitude')$data), 
    c(longitude = 10, latitude = 2)
  )
  expect_equal(
    dim(SelBox(dat3, lat = lat3, lon = lon3, region = region3, 
               londim = 'longitude', latdim = 'latitude', mask = mask3)$mask), 
    c(latitude = 2, longitude = 10)
  )
  expect_equal(
    SelBox(dat3, lat = lat3, lon = lon3, region = region3, 
           londim = 'longitude', latdim = 'latitude', mask = mask3)$data, 
    array(c(1:10, 21:30), dim = c(longitude = 10, latitude = 2))
  )
  expect_equal(
    SelBox(dat3, lat = lat3, lon = lon3, region = region3, 
           londim = 'longitude', latdim = 'latitude', mask = mask3)$lon, 
    seq(2, 20, 2)
  )
  expect_equal(
    SelBox(dat3, lat = lat3, lon = lon3, region = region3, 
           londim = 'longitude', latdim = 'latitude', mask = mask3)$lat, 
    c(1, 5)
  )
  expect_equal(
    SelBox(dat3, lat = lat3, lon = lon3, region = region3_2,
           londim = 'longitude', latdim = 'latitude', mask = mask3)$lon,
    c(seq(16, 40, 2), 2, 4)
  )
})
